# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import os

from ._util import (
    check_types, error_to_str, guess_type, ffi, lib, get_encoded_len, Type,
    NoneType,
)
from .exceptions import VerificationError, HashingError


__all__ = [
    "DEFAULT_RANDOM_SALT_LENGTH",
    "hash_password",
    "hash_password_raw",
    "verify_password",
]

DEFAULT_RANDOM_SALT_LENGTH = 16
DEFAULT_HASH_LENGTH = 16
DEFAULT_TIME_COST = 3
DEFAULT_MEMORY_COST = 2**12
DEFAULT_PARALLELISM = 2


def hash_password(password, salt=None,
                  time_cost=DEFAULT_TIME_COST,
                  memory_cost=DEFAULT_MEMORY_COST,
                  parallelism=DEFAULT_PARALLELISM,
                  hash_len=DEFAULT_HASH_LENGTH,
                  type=Type.I):
    """
    Hash *password* and return an *encoded* hash.
    """
    return _hash(password, salt, time_cost, memory_cost, parallelism, hash_len,
                 type, True)


def hash_password_raw(password, salt=None,
                      time_cost=DEFAULT_TIME_COST,
                      memory_cost=DEFAULT_MEMORY_COST,
                      parallelism=DEFAULT_PARALLELISM,
                      hash_len=DEFAULT_HASH_LENGTH,
                      type=Type.I):
    """
    Hash *password* and return a *raw* hash.
    """
    return _hash(password, salt, time_cost, memory_cost, parallelism, hash_len,
                 type, False)


def _hash(password, salt, time_cost, memory_cost, parallelism, hash_len, type,
          encoded):
    e = check_types(
        password=(password, bytes),
        salt=(salt, (bytes, NoneType)),
        time_cost=(time_cost, int),
        memory_cost=(memory_cost, int),
        parallelism=(parallelism, int),
        hash_len=(hash_len, int),
        type=(type, Type),
        encoded=(encoded, bool),
    )
    if e:
        raise TypeError(e)
    if salt is None:
        salt = os.urandom(DEFAULT_RANDOM_SALT_LENGTH)

    raw_buf = encoded_buf = ffi.NULL
    raw_len = encoded_len = 0
    if encoded:
        encoded_len = get_encoded_len(hash_len, len(salt))
        encoded_buf = ffi.new("char[]", encoded_len)
    else:
        raw_len = hash_len
        raw_buf = ffi.new("char[]", raw_len)

    rv = lib.argon2_hash(
        time_cost, memory_cost, parallelism,
        ffi.new("char[]", password), len(password),
        ffi.new("char[]", salt), len(salt),
        raw_buf, hash_len,
        encoded_buf, encoded_len,
        type.value,
    )
    if rv != lib.ARGON2_OK:
        raise HashingError(error_to_str(rv))

    return (
        ffi.string(encoded_buf) if encoded_len != 0
        else bytes(ffi.buffer(raw_buf))
    )


def verify_password(hash, password, type=None):
    """
    Verify whether *password* is correct for *hash* of *type*.

    :return: ``True`` on success, throw exception otherwise.
    :rtype: bool
    """
    e = check_types(
        password=(password, bytes),
        hash=(hash, bytes),
        type=(type, (Type, NoneType)),
    )
    if e:
        raise TypeError(e)

    if type is None:
        type = guess_type(hash)

    rv = lib.argon2_verify(
        ffi.new("char[]", hash),
        ffi.new("char[]", password),
        len(password),
        type.value,
    )
    if rv == lib.ARGON2_OK:
        return True
    else:
        raise VerificationError(error_to_str(rv))



#!/usr/bin/env python

from ctypes import cdll, c_uint32, c_char_p, c_size_t, c_int, create_string_buffer
import os
import platform
import sys


SYSTEM_TYPE = platform.system()
IS_PY2 = sys.version_info < (3, 0, 0, 'final', 0)
LOCAL_LIB_PATH = "../../"

if SYSTEM_TYPE == "Linux":
    LIB_NAME = "libargon2.so"
elif SYSTEM_TYPE == "Windows":
    LIB_NAME = "libargon2.dll"
elif SYSTEM_TYPE == "Darwin":
    LIB_NAME = "libargon2.dylib"
else:
    raise OSError("Your OS is not fully supported, you may try to modify code manually")

try:
    GLOBAL_LIB_PATH = os.environ['LD_LIBRARY_PATH']
    _argon2 = cdll.LoadLibrary(GLOBAL_LIB_PATH + LIB_NAME)
except (OSError, KeyError) as e:
    try:
        _argon2 = cdll.LoadLibrary(LOCAL_LIB_PATH + LIB_NAME)
    except OSError as e:
        raise OSError("You haven't installed Argon2 lib")

_crypto_argon2 = _argon2.argon2_hash
_crypto_argon2.argtypes = [
    c_uint32,  # uint32_t       N
    c_uint32,  # uint32_t       r
    c_uint32,  # uint32_t       p

    c_char_p,  # const uint8_t *passwd
    c_size_t,  # size_t         passwdlen

    c_char_p,  # const uint8_t *salt
    c_size_t,  # size_t         saltlen

    c_char_p,  # uint8_t       *buf
    c_size_t,  # size_t         buflen

    c_char_p,  # uint8_t       *encoded
    c_size_t,  # size_t         encodedlen

    c_uint32,  # uint32_t       Argon_type
]
_crypto_argon2.restype = c_int


class Argon2Type(object):
    Argon2_d = 0
    Argon2_i = 1


class Argon2Exception(Exception):
    errors = ("ARGON2_OK", "ARGON2_OUTPUT_PTR_NULL", "ARGON2_OUTPUT_TOO_SHORT", "ARGON2_OUTPUT_TOO_LONG",
              "ARGON2_PWD_TOO_SHORT", "ARGON2_PWD_TOO_LONG", "ARGON2_SALT_TOO_SHORT",
              "ARGON2_SALT_TOO_LONG", "ARGON2_AD_TOO_SHORT", "ARGON2_AD_TOO_LONG",
              "ARGON2_SECRET_TOO_SHORT", "ARGON2_SECRET_TOO_LONG",
              "ARGON2_TIME_TOO_SMALL", "ARGON2_TIME_TOO_LARGE", "ARGON2_MEMORY_TOO_LITTLE",
              "ARGON2_MEMORY_TOO_MUCH", "ARGON2_LANES_TOO_FEW", "ARGON2_LANES_TOO_MANY",
              "ARGON2_PWD_PTR_MISMATCH", "ARGON2_SALT_PTR_MISMATCH", "ARGON2_SECRET_PTR_MISMATCH",
              "ARGON2_AD_PTR_MISMATCH", "ARGON2_MEMORY_ALLOCATION_ERROR", "ARGON2_FREE_MEMORY_CBK_NULL",
              "ARGON2_ALLOCATE_MEMORY_CBK_NULL", "ARGON2_INCORRECT_PARAMETER", "ARGON2_INCORRECT_TYPE",
              "ARGON2_OUT_PTR_MISMATCH", "ARGON2_THREADS_TOO_FEW", "ARGON2_THREADS_TOO_MANY",
              "ARGON2_MISSING_ARGS", "ARGON2_ENCODING_FAIL", "ARGON2_DECODING_FAIL")


def _ensure_bytes(data):
    if IS_PY2 and isinstance(data, unicode):
        raise TypeError('can not encrypt/decrypt unicode objects')

    if not IS_PY2 and isinstance(data, str):
        return bytes(data, 'utf-8')

    return data


def argon2_hash(password, salt, t=16, m=8, p=1, buflen=128, argon_type=Argon2Type.Argon2_i):
    outbuf = create_string_buffer(buflen)
    password = _ensure_bytes(password)
    salt = _ensure_bytes(salt)

    result = _crypto_argon2(t, m, p,
                            password, len(password),
                            salt, len(salt),
                            outbuf, buflen,
                            None, 0,
                            argon_type)

    if result:
        raise Argon2Exception(Argon2Exception.errors[result])

    return outbuf.raw


if __name__ == "__main__":
    print(argon2_hash("password", "some salt"))



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from enum import Enum
from six import iteritems, PY3


from ._ffi import ffi, lib
from .exceptions import InvalidHash


class Type(Enum):
    D = lib.Argon2_d
    I = lib.Argon2_i


NoneType = type(None)


def check_types(**kw):
    """
    Check each ``name: (value, types)`` in *kw*.

    Returns a human-readable string of all violations or `None``.
    """
    errors = []
    for name, (value, types) in iteritems(kw):
        if not isinstance(value, types):
            if isinstance(types, tuple):
                types = ", or ".join(t.__name__ for t in types)
            else:
                types = types.__name__
            errors.append("'{name}' must be a {type} (got {actual})".format(
                name=name,
                type=types,
                actual=type(value).__name__,
            ))

    if errors != []:
        return ", ".join(errors) + "."


def error_to_str(error):
    """
    Convert an Argon2 error code into a native string.
    """
    msg = ffi.string(lib.error_message(error))
    if PY3:
        msg = msg.decode("ascii")
    return msg


def encoded_str_len(l):
    """
    Compute how long a byte string of length *l* becomes if encoded to hex.
    """
    return (l << 2) / 3 + 2


def get_encoded_len(hash_len, salt_len):
    """
    Compute the size of the required buffer for an encoded hash with *hash_len*
    and *salt_len*.
    """
    # From https://github.com/P-H-C/phc-winner-argon2/blob/master/src/run.c:
    #
    # Sample encode: $argon2i$m=65536,t=2,p=4$c29tZXNhbHQAAAAAAAAAAA$QWLzI4TY9H
    # kL2ZTLc8g6SinwdhZewYrzz9zxCo0bkGY
    # Maximumum lengths are defined as:
    # strlen $argon2i$ = 9
    # m=65536 with strlen (uint32_t)-1 = 10, so this total is 12
    # ,t=2,p=4 If we consider each number to potentially reach four digits
    # in future, this = 14
    # $c29tZXNhbHQAAAAAAAAAAA Formula for this is
    # (SALT_LEN * 4 + 3) / 3 + 1 = 23
    # $QWLzI4TY9HkL2ZTLc8g6SinwdhZewYrzz9zxCo0bkGY per above formula, = 44
    # + NULL byte
    # 9 + 12 + 14 + 23 + 44 + 1 = 103
    # Rounded to 4 byte boundary: 104
    return (42 + int(((hash_len + salt_len) << 2) / 3)) & ~3


def guess_type(s):
    """
    Guesses what type of encoded Argon2 hash *s* is or raises InvalidHash.
    """
    prefix = s[:8]
    if prefix == b"$argon2i":
        return Type.I
    elif prefix == b"$argon2d":
        return Type.D
    else:
        raise InvalidHash(s)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import binascii

import pytest

from hypothesis import given
from hypothesis import strategies as st

from argon2 import (
    hash_password, hash_password_raw, verify_password, Type,
    DEFAULT_RANDOM_SALT_LENGTH,
)
from argon2.exceptions import VerificationError, HashingError
from argon2._util import encoded_str_len

# Example data obtained using the official Argon2 CLI client:
#
# $ echo -n "password" | ./argon2 somesalt -t 2 -m 16 -p 4
# Type:       Argon2i
# Iterations: 2
# Memory:     65536 KiB
# Parallelism:    4
# Hash:       4162f32384d8f4790bd994cb73c83a4a29f076165ec18af3cfdcf10a8d1b9066
# Encoded:    $argon2i$m=65536,t=2,p=4$c29tZXNhbHQAAAAAAAAAAA$QWLzI4TY9HkL2ZTLc
#             8g6SinwdhZewYrzz9zxCo0bkGY
# 0.176 seconds
# Verification ok
#
# $ echo -n "password" | ./argon2 somesalt -t 2 -m 16 -p 4 -d
# Type:       Argon2d
# Iterations: 2
# Memory:     65536 KiB
# Parallelism:    4
# Hash:       9ca3b9fc007d09daf489dcf854e9a785ff5a32c62ec50acf26477977add23225
# Encoded:    $argon2d$m=65536,t=2,p=4$c29tZXNhbHQAAAAAAAAAAA$nKO5/AB9Cdr0idz4V
#             Omnhf9aMsYuxQrPJkd5d63SMiU
# 0.189 seconds
# Verification ok

TEST_HASH_I = (
    b"$argon2i$m=65536,t=2,p=4"
    b"$c29tZXNhbHQAAAAAAAAAAA"
    b"$QWLzI4TY9HkL2ZTLc8g6SinwdhZewYrzz9zxCo0bkGY"
)
TEST_HASH_D = (
    b"$argon2d$m=65536,t=2,p=4"
    b"$c29tZXNhbHQAAAAAAAAAAA$"
    b"nKO5/AB9Cdr0idz4VOmnhf9aMsYuxQrPJkd5d63SMiU"
)
TEST_RAW_I = binascii.unhexlify(
    b"4162f32384d8f4790bd994cb73c83a4a29f076165ec18af3cfdcf10a8d1b9066"
)
TEST_RAW_D = binascii.unhexlify(  # N.B. includes NUL byte!
    b"9ca3b9fc007d09daf489dcf854e9a785ff5a32c62ec50acf26477977add23225"
)
TEST_HASH_FAST = (
    b"$argon2i$m=8,t=1,p=1$c29tZXNhbHQAAAAAAAAAAA$owd7NH5aC7mrx3sIc0zMF+R8RkPH"
    b"S23ZuFM0IO3uck8"
)  # same password/salt, but much cheaper.

TEST_PASSWORD = b"password"
TEST_SALT_LEN = 16
TEST_SALT = b"somesalt"
TEST_SALT += b"\x00" * (TEST_SALT_LEN - len(TEST_SALT))
TEST_TIME = 2
TEST_MEMORY = 65536
TEST_PARALLELISM = 4
TEST_HASH_LEN = 32

i_and_d_encoded = pytest.mark.parametrize("type,hash", [
    (Type.I, TEST_HASH_I,),
    (Type.D, TEST_HASH_D,),
])
i_and_d_raw = pytest.mark.parametrize("type,hash", [
    (Type.I, TEST_RAW_I,),
    (Type.D, TEST_RAW_D,),
])


class TestHash(object):
    @i_and_d_encoded
    def test_hash_password(self, type, hash):
        """
        Creates the same encoded hash as the Argon2 CLI client.
        """
        rv = hash_password(
            TEST_PASSWORD,
            TEST_SALT,
            TEST_TIME,
            TEST_MEMORY,
            TEST_PARALLELISM,
            TEST_HASH_LEN,
            type,
        )

        assert hash == rv
        assert isinstance(rv, bytes)

    @i_and_d_raw
    def test_hash_password_raw(self, type, hash):
        """
        Creates the same raw hash as the Argon2 CLI client.
        """
        rv = hash_password_raw(
            TEST_PASSWORD,
            TEST_SALT,
            TEST_TIME,
            TEST_MEMORY,
            TEST_PARALLELISM,
            TEST_HASH_LEN,
            type,
        )

        assert hash == rv
        assert isinstance(rv, bytes)

    def test_hash_nul_bytes(self):
        """
        Hashing passwords with NUL bytes works as expected.
        """
        rv = hash_password_raw(b"abc\x00", TEST_SALT)

        assert rv != hash_password_raw(b"abc", TEST_SALT)

    def test_random_salt(self):
        """
        Omitting a salt, creates a random one.
        """
        rv = hash_password(b"secret")
        salt = rv.split(b",")[-1].split(b"$")[1]
        assert (
            # -1 for not NUL byte
            int(encoded_str_len(DEFAULT_RANDOM_SALT_LENGTH)) - 1 == len(salt)
        )

    def test_hash_wrong_arg_type(self):
        """
        Passing an argument of wrong type raises TypeError.
        """
        with pytest.raises(TypeError):
            hash_password(u"oh no, unicode!")

    def test_illegal_argon2_parameter(self):
        """
        Raises HashingError if hashing fails.
        """
        with pytest.raises(HashingError):
            hash_password(TEST_PASSWORD, memory_cost=1)

    @given(st.binary(max_size=128))
    def test_hash_fast(self, password):
        """
        Hash various passwords as cheaply as possible.
        """
        hash_password(
            password,
            salt=b"12345678",
            time_cost=1,
            memory_cost=8,
            parallelism=1,
            hash_len=8,
        )


class TestVerify(object):
    @i_and_d_encoded
    def test_auto_success(self, type, hash):
        """
        Given a valid hash and password, we figure out the type ourself and
        succeed.
        """
        assert True is verify_password(hash, TEST_PASSWORD)

    @i_and_d_encoded
    def test_explicit_success(self, type, hash):
        """
        Given a valid hash and password and correct type, we succeed.
        """
        assert True is verify_password(hash, TEST_PASSWORD, type)

    def test_explicit_fail(self):
        """
        Given a valid hash and password and wrong type, we fail.
        """
        with pytest.raises(VerificationError):
            verify_password(TEST_HASH_I, TEST_PASSWORD, Type.D)

    def test_wrong_arg_type(self):
        """
        Passing an argument of wrong type raises TypeError.
        """
        with pytest.raises(TypeError):
            verify_password(TEST_HASH_I, TEST_PASSWORD.decode("ascii"))

    def test_auto_fast(self):
        """
        Just a test ensure that test_auto_fail works on correct data.
        """
        assert verify_password(TEST_HASH_FAST, b"password")

    @given(st.binary(max_size=128))
    def test_auto_fail(self, wrong_password):
        """
        Given a valid but wrong hash and password, we figure out the type
        ourself and fail.
        """
        with pytest.raises(VerificationError):
            verify_password(TEST_HASH_FAST, wrong_password)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import sys

from cffi import FFI


include_dirs = ["libargon2/src"]

if sys.version_info[0] == 2 and "win32" in str(sys.platform).lower():
    include_dirs.append("extras/msinttypes")


ffi = FFI()
ffi.set_source(
    "_ffi", "#include <argon2.h>",
    include_dirs=include_dirs,
    libraries=["libargon2"],
)

ffi.cdef("""\
typedef enum Argon2_type { Argon2_d = 0, Argon2_i = 1 } argon2_type;

int argon2_hash(const uint32_t t_cost, const uint32_t m_cost,
                const uint32_t parallelism, const void *pwd,
                const size_t pwdlen, const void *salt, const size_t saltlen,
                void *hash, const size_t hashlen, char *encoded,
                const size_t encodedlen, argon2_type type);

int argon2_verify(const char *encoded, const void *pwd, const size_t pwdlen,
                  argon2_type type);

const char *error_message(int error_code);

/* Error codes */
typedef enum Argon2_ErrorCodes {
    ARGON2_OK = 0,

    ARGON2_OUTPUT_PTR_NULL = 1,

    ARGON2_OUTPUT_TOO_SHORT = 2,
    ARGON2_OUTPUT_TOO_LONG = 3,

    ARGON2_PWD_TOO_SHORT = 4,
    ARGON2_PWD_TOO_LONG = 5,

    ARGON2_SALT_TOO_SHORT = 6,
    ARGON2_SALT_TOO_LONG = 7,

    ARGON2_AD_TOO_SHORT = 8,
    ARGON2_AD_TOO_LONG = 9,

    ARGON2_SECRET_TOO_SHORT = 10,
    ARGON2_SECRET_TOO_LONG = 11,

    ARGON2_TIME_TOO_SMALL = 12,
    ARGON2_TIME_TOO_LARGE = 13,

    ARGON2_MEMORY_TOO_LITTLE = 14,
    ARGON2_MEMORY_TOO_MUCH = 15,

    ARGON2_LANES_TOO_FEW = 16,
    ARGON2_LANES_TOO_MANY = 17,

    ARGON2_PWD_PTR_MISMATCH = 18,    /* NULL ptr with non-zero length */
    ARGON2_SALT_PTR_MISMATCH = 19,   /* NULL ptr with non-zero length */
    ARGON2_SECRET_PTR_MISMATCH = 20, /* NULL ptr with non-zero length */
    ARGON2_AD_PTR_MISMATCH = 21,     /* NULL ptr with non-zero length */

    ARGON2_MEMORY_ALLOCATION_ERROR = 22,

    ARGON2_FREE_MEMORY_CBK_NULL = 23,
    ARGON2_ALLOCATE_MEMORY_CBK_NULL = 24,

    ARGON2_INCORRECT_PARAMETER = 25,
    ARGON2_INCORRECT_TYPE = 26,

    ARGON2_OUT_PTR_MISMATCH = 27,

    ARGON2_THREADS_TOO_FEW = 28,
    ARGON2_THREADS_TOO_MANY = 29,

    ARGON2_MISSING_ARGS = 30,

    ARGON2_ENCODING_FAIL = 31,

    ARGON2_DECODING_FAIL = 32,

    ARGON2_ERROR_CODES_LENGTH /* Do NOT remove; Do NOT add error codes after
                                 this
                                 error code */
} argon2_error_codes;
""")

if __name__ == '__main__':
    ffi.compile()



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function


class Argon2Error(Exception):
    """
    Superclass of all argon2 exceptions.

    Never thrown directly.
    """


class VerificationError(Argon2Error):
    """
    Raised if verification failed.
    """


class InvalidHash(VerificationError):
    """
    Raised if :func:`argon2.verify_password` should guess the type of a hash
    but the hash is invalid.
    """


class HashingError(Argon2Error):
    """
    Raised if hasing failed.
    """



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from six import PY3

import pytest

from hypothesis import given
from hypothesis import strategies as st

from argon2.exceptions import InvalidHash
from argon2._util import get_encoded_len, check_types, NoneType, guess_type

from .test_api import i_and_d_encoded


def test_get_encoded_len():
    """
    Verify we get the same result as the official example.
    """
    assert 104 == get_encoded_len(32, 16)


class TestCheckTypes(object):
    def test_success(self):
        """
        Returns None if all types are okay.
        """
        assert None is check_types(
            bytes=(b"bytes", bytes),
            tuple=((1, 2), tuple),
            str_or_None=(None, (str, NoneType)),
        )

    def test_fail(self):
        """
        Returns summary of failures.
        """
        rv = check_types(
            bytes=(u"not bytes", bytes),
            str_or_None=(42, (str, NoneType))
        )

        assert "." == rv[-1]  # proper grammar FTW
        assert "'str_or_None' must be a str, or NoneType (got int)" in rv

        if PY3:
            assert "'bytes' must be a bytes (got str)" in rv
        else:
            assert "'bytes' must be a str (got unicode)" in rv


class TestGuessType(object):
    @i_and_d_encoded
    def test_success(self, type, hash):
        """
        Returns Type.I on likely Argon2i hashes.
        """
        assert type is guess_type(hash)

    @given(st.binary(max_size=128))
    def test_fail(self, wrong_hash):
        """
        If the hash can't be neither i nor d, raise InvalidHash.
        """
        with pytest.raises(InvalidHash):
            guess_type(wrong_hash)



from setuptools import setup, find_packages

import sys
import codecs
import os
import re


HERE = os.path.abspath(os.path.dirname(__file__))

###############################################################################

NAME = "argon2_cffi"
PACKAGES = find_packages(where="src")
CFFI_MODULES = ["src/argon2/_ffi_build.py:ffi"]
include_dirs = [
    "libargon2/src",
    "libargon2/src/blake2",
]

if sys.version_info[0] == 2 and "win32" in str(sys.platform).lower():
    include_dirs.append("extras/msinttypes")

LIBRARIES = [
    ("libargon2", {
        "include_dirs": include_dirs,
        "sources": [
            "libargon2/src/argon2.c",
            "libargon2/src/blake2/blake2b.c",
            "libargon2/src/core.c",
            "libargon2/src/encoding.c",
            "libargon2/src/opt.c",
            "libargon2/src/ref.c",
            "libargon2/src/thread.c",
        ],
    }),
]
META_PATH = ("src", "argon2", "__init__.py")
KEYWORDS = ["password", "hash", "hashing", "security"]
CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Programming Language :: Python",
    "Topic :: Software Development :: Libraries :: Python Modules",
]

SETUP_REQUIRES = ["cffi"]
INSTALL_REQUIRES = ["six", "cffi>=1.0.0"]
EXTRAS_REQUIRE = {}
if sys.version_info[0:2] < (3, 4):  # old school
    INSTALL_REQUIRES += ["enum34"]

###############################################################################


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


META_FILE = read(*META_PATH)


def find_meta(meta):
    """
    Extract __*meta*__ from META_FILE.
    """
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=meta),
        META_FILE, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))


if __name__ == "__main__":
    setup(
        name=NAME,
        description=find_meta("description"),
        license=find_meta("license"),
        url=find_meta("uri"),
        version=find_meta("version"),
        author=find_meta("author"),
        author_email=find_meta("email"),
        maintainer=find_meta("author"),
        maintainer_email=find_meta("email"),
        long_description=(
            read("README.rst") + "\n\n" +
            read("AUTHORS.rst") + "\n\n" +
            read("CHANGELOG.rst")
        ),
        keywords=KEYWORDS,
        packages=PACKAGES,
        package_dir={"": "src"},
        cffi_modules=CFFI_MODULES,
        ext_package="argon2",
        libraries=LIBRARIES,
        classifiers=CLASSIFIERS,
        setup_requires=SETUP_REQUIRES,
        install_requires=INSTALL_REQUIRES,
        extras_require=EXTRAS_REQUIRE,
        zip_safe=False,
    )
